<?php declare(strict_types=1);

namespace Everyware\Plugin\CacheInvalidator;

/**
 * Class DataStorage
 * @package Everyware\Plugin\CacheInvalidator
 */
class DataStorage
{
    private $tableName = 'ew_cache_invalidation';

    /**
     * DataStorage constructor.
     */
    public function __construct()
    {
        $this->setupDatabase();
    }

    /**
     * Setup database, create table if it exists.
     */
    private function setupDatabase(): void
    {
        if ($this->databaseExist()) {
            return;
        }

        global $wpdb;
        $table_name = $wpdb->prefix . $this->tableName;
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE $table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            oc_query TEXT NOT NULL,
            page_id int NULL,
            page_url VARCHAR(512) NULL,
            type VARCHAR(128),
            listener_id VARCHAR(512),
            UNIQUE KEY id (id)
        ) $charset_collate;";

        require_once ABSPATH . 'wp-admin/includes/upgrade.php';
        dbDelta($sql);
    }

    /**
     * Check if database table exists.
     *
     * @return bool
     */
    private function databaseExist(): bool
    {

        global $wpdb;
        $tableName = $wpdb->prefix . $this->tableName;

        return $wpdb->get_var("SHOW TABLES LIKE '" . $tableName . "'") !== null;
    }

    /**
     * Save a query item to database.
     *
     * @param \Everyware\Plugin\CacheInvalidator\QueryItem $item
     *
     * @return bool|int
     */
    public function saveItem(QueryItem $item)
    {

        global $wpdb;

        if ($item->id > 0) {

            $result = $wpdb->query(
                $wpdb->prepare("UPDATE {$wpdb->prefix}{$this->tableName}
                      SET oc_query = %s, page_id = %d, page_url = %s, type = %s, listener_id = %s
                      WHERE id = %d", $item->ocQuery, $item->pageId, $item->pageUrl, $item->type, $item->listenerId,
                    $item->id)
            );

            if ($result !== false) {
                return $item->id;
            }
        } else {


            $result = $wpdb->query(
                $wpdb->prepare("INSERT INTO {$wpdb->prefix}{$this->tableName} (oc_query, page_id, page_url, type, listener_id)
                      VALUES (%s, %d, %s, %s, %s)", $item->ocQuery, $item->pageId, $item->pageUrl, $item->type,
                    $item->listenerId)
            );

            if ($result !== false) {
                return $wpdb->insert_id;
            }
        }

        return false;
    }

    /**
     * Get all query items from database.
     *
     * @return array|bool
     */
    public function getAllItems()
    {
        global $wpdb;
        $result = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}{$this->tableName} ORDER BY page_id, page_url");

        if ($result !== false) {

            $queryItemArr = [];
            foreach ($result as $item) {

                if ($item->page_id === '0' && strlen($item->page_url) > 0) {
                    $queryItemArr[] = new QueryItem($item->oc_query, $item->page_url, $item->type, $item->listener_id,
                        $item->id);
                } else {
                    $queryItemArr[] = new QueryItem($item->oc_query, (int)$item->page_id, $item->type,
                        $item->listener_id, $item->id);
                }
            }

            return $queryItemArr;
        }

        return false;
    }

    /**
     * Get query items by page id.
     *
     * @param $page_id
     *
     * @return array|bool
     */
    public function getItemsByPageId($page_id)
    {

        global $wpdb;
        $result = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}{$this->tableName} WHERE page_id = {$page_id}");

        if ($result !== false) {

            $queryItemArr = [];
            foreach ($result as $item) {

                if ($item->page_id === "0" && strlen($item->page_url) > 0) {
                    $queryItemArr[] = new QueryItem($item->oc_query, $item->page_url, $item->type, $item->listener_id,
                        $item->id);
                } else {
                    $queryItemArr[] = new QueryItem($item->oc_query, (int)$item->page_id, $item->type,
                        $item->listener_id, $item->id);
                }
            }

            return $queryItemArr;
        }

        return false;
    }

    /**
     * Get query items by listener id.
     *
     * @param $listener_id
     *
     * @return array|bool
     */
    public function getItemsByListenerId($listener_id)
    {
        global $wpdb;
        $result = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}{$this->tableName} WHERE listener_id = \"{$listener_id}\"");

        if ($result !== false) {

            $queryItemArr = [];
            foreach ($result as $item) {

                if ($item->page_id === '0' && strlen($item->page_url) > 0) {
                    $queryItemArr[] = new QueryItem($item->oc_query, $item->page_url, $item->type, $item->listener_id,
                        $item->id);
                } else {
                    $queryItemArr[] = new QueryItem($item->oc_query, (int)$item->page_id, $item->type,
                        $item->listener_id, $item->id);
                }
            }

            return $queryItemArr;
        }

        return false;
    }

    /**
     * Delete all items for given page.
     *
     * @param $pageId
     */
    public function deleteItemsByPageId($pageId): void
    {

        global $wpdb;
        $wpdb->delete($wpdb->prefix . $this->tableName, ['page_id' => $pageId], ['%d']);
    }

    /**
     * Delete item with given id.
     *
     * @param $id
     */
    public function deleteItem($id): void
    {
        global $wpdb;
        $wpdb->delete($wpdb->prefix . $this->tableName, ['id' => $id], ['%d']);
    }

    /**
     * Purge table, remove all items.
     *
     * @return bool
     */
    public function purgeTable(): bool
    {
        global $wpdb;
        $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}{$this->tableName}");

        return true;
    }
}
